@@ -9,6 +9,8 @@ use strict;
 use POSIX qw/ locale_h strftime /;
 use Encode qw/ encode from_to decode /;
 use Encode::Guess qw/ euc-jp shiftjis /;
+use File::Copy;
+use File::Path;
 use Data::Dumper;
 
 #=======================================================
@@ -22,9 +24,9 @@ my @gnReqLibs = ("Compress::Zlib",	"Config",		"Data::Dumper",	"Encode",
                          "threads",		"threads::shared",	"utf8");
 my $gnUserLang;
 my $gnPerlpath;
-my $binpath = '/usr/bin/';
-my $sharepath = '/usr/share/';
-my $manpath = '/usr/share/man/';
+my $binpath = &gnGetBinPath;
+my $sharepath = &gnGetLibPath;
+my $manpath = $sharepath . "man\/";
 
 for(;;) {
 # Step 1 (日本語が出力可能であることをチェック)=======================
@@ -46,7 +48,8 @@ for(;;) {
    if($stepflg == 2) {
       &gnprint("==================================\ngnviewで使用するperlを指定します\n");
       
-      my @pl = `which -a perl`;
+#      my @pl = `which -a perl 2\>/dev/null`;
+      my @pl = &gnWhich(); 
       my $b=0;
       foreach my $a (@pl) {
          chomp($a);
@@ -88,7 +91,7 @@ for(;;) {
          }
       }
       if($cnt3_1 == 0) {
-         &gnprint("\[ERROR\]指定されたperlはithreadsをサポートしていません");
+         &gnprint("\[ERROR\]指定されたperlはithreadsをサポートしていません\n");
          &gnprint("\[ERROR\]gnviewの動作にはithreadsに対応したperlを用意する必要があります\n");
          $stepflg--;
       }else{
@@ -169,15 +172,24 @@ for(;;) {
                last;
             }else{
                &gnprint("実行ファイルパスを指定してください\[${binpath}\] ");
-               ${binpath} = <STDIN>; chomp(${binpath});
+               my ${newbinpath} = <STDIN>; chomp(${newbinpath});
+               if($newbinpath ne "") {
+                  $binpath = $newbinpath;
+               }
                my $aaa = substr($binpath,-1,1);
                if($aaa ne "\/") { $binpath = $binpath . "\/"; }
                &gnprint("共有ライブラリパスを指定してください\[${sharepath}\] ");
-               ${sharepath} = <STDIN>; chomp(${sharepath});
+               my ${newsharepath} = <STDIN>; chomp(${newsharepath});
+               if($newsharepath ne "") {
+                  $sharepath = $newsharepath;
+               }
                $aaa = substr($sharepath,-1,1);
                if($aaa ne "\/") { $sharepath = $sharepath . "\/"; }
                &gnprint("マニュアルパスを指定してください\[${manpath}\] ");
-               ${manpath} = <STDIN>; chomp(${manpath});
+               my ${newmanpath} = <STDIN>; chomp(${newmanpath});
+               if($newmanpath ne "") {
+                  $manpath = $newmanpath;
+               }
                $aaa = substr($manpath,-1,1);
                if($aaa ne "\/") { $manpath = $manpath . "\/"; }
             }
@@ -209,31 +221,83 @@ for(;;) {
 
       my $uninstinfo = $ENV{'HOME'} . "\/.gnview.uninstinfo";
       open(gnUN, ">$uninstinfo");
-      my $cmd;
-      $cmd = "install -D -d $binpath"; system("$cmd");
-      $cmd = "install -m 0755 gnview ${binpath}gnview"; system("$cmd");
+#      my $cmd;
+      my $dir;
+      my $dirret;
+      my @instfile;
+#      $cmd = "install -D -d $binpath"; system("$cmd");
+      $dir = $binpath;
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0755 gnview ${binpath}gnview"; system("$cmd");
+      @instfile = ("gnview");
+      &gnInstall(@instfile, "0755", $dir);
       print gnUN "${binpath}gnview\n";
       
-      $cmd = "install -D -d ${sharepath}applications"; system("$cmd");
-      $cmd = "install -m 0644 gnview.desktop ${sharepath}applications"; system("$cmd");
+#      $cmd = "install -D -d ${sharepath}applications"; system("$cmd");
+      $dir = "${sharepath}applications";
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0644 gnview.desktop ${sharepath}applications"; system("$cmd");
+      @instfile = ("gnview.desktop");
+      &gnInstall(@instfile, "0644", $dir);
       print gnUN "${sharepath}applications\/gnview.desktop\n";
 
-      $cmd = "install -D -d ${sharepath}pixmaps"; system("$cmd");
-      $cmd = "install -m 0644 gnview.png ${sharepath}pixmaps"; system("$cmd");
+#      $cmd = "install -D -d ${sharepath}pixmaps"; system("$cmd");
+      $dir = "${sharepath}pixmaps";
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0644 gnview.png ${sharepath}pixmaps"; system("$cmd");
+      @instfile = ("gnview.png");
+      &gnInstall(@instfile, "0644", $dir);
       print gnUN "${sharepath}pixmaps\/gnview.png\n";
 
-      $cmd = "install -D -d ${sharepath}doc\/gnview"; system("$cmd");
-      $cmd = "install -m 0644 AUTHORS  COPYING  ChangeLog  NEWS  README  TODO ${sharepath}doc\/gnview"; system("$cmd");
+#      $cmd = "install -D -d ${sharepath}doc\/gnview"; system("$cmd");
+      $dir = "${sharepath}doc\/gnview";
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0644 AUTHORS  COPYING  ChangeLog  NEWS  README  TODO ${sharepath}doc\/gnview"; system("$cmd");
+      @instfile = ("AUTHORS", "COPYING", "ChangeLog", "NEWS", "README", "TODO");
+      &gnInstall(@instfile, "0644", $dir);
       print gnUN "${sharepath}doc\/gnview\n";
 
-      $cmd = "install -D -d ${sharepath}gnview"; system("$cmd");
-      $cmd = "install -m 0644 gnview_firstrun.pl gnview_env.pl ${sharepath}gnview"; system("$cmd");
-      $cmd = "install -m 0644 1pix.png 1pix_glay.png gnview.png ${sharepath}gnview"; system("$cmd");
-      $cmd = "install -m 0644 gnview.glade gikoNavi.ini url.ini sent.ini ${sharepath}gnview"; system("$cmd");
+#      $cmd = "install -D -d ${sharepath}gnview"; system("$cmd");
+      $dir = "${sharepath}gnview";
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0644 gnview_firstrun.pl gnview_env.pl ${sharepath}gnview"; system("$cmd");
+#      $cmd = "install -m 0644 1pix.png 1pix_glay.png gnview.png ${sharepath}gnview"; system("$cmd");
+#      $cmd = "install -m 0644 gnview.glade gikoNavi.ini url.ini sent.ini ${sharepath}gnview"; system("$cmd");
+      @instfile = ("gnview_firstrun.pl", "gnview_env.pl", "1pix.png", "1pix_glay.png", "gnview.png",
+                      "gnview.glade", "gikoNavi.ini", "url.ini", "sent.ini");
+      &gnInstall(@instfile, "0644", $dir);
       print gnUN "${sharepath}gnview\n";
 
-      $cmd = "install -D -d ${manpath}man1"; system("$cmd");
-      $cmd = "install -m 0644 gnview.1.gz ${manpath}man1"; system("$cmd");
+#      $cmd = "install -D -d ${manpath}man1"; system("$cmd");
+      $dir = "${manpath}man1";
+      $dirret = &gnMkdir($dir, "0755");
+      if($dirret>2) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+#      $cmd = "install -m 0644 gnview.1.gz ${manpath}man1"; system("$cmd");
+      @instfile = ("gnview.1.gz");
+      &gnInstall(@instfile, "0644", $dir);
       print gnUN "${manpath}man1\/gnview.1.gz\n";
       
       close(gnUN);
@@ -250,6 +314,123 @@ if($stepflg > 5) {
 };
 # 内部関数 ================================================
 
+sub gnGetBinPath {
+   # 実行ファイルパスをチェック
+   #
+   # 引数:      なし
+   # 返り値: 1: 実行ファイルへのフルパス
+   my $gnBinPath;
+      if($ENV{'PATH'}) {
+         my @dvars = split(/:/, $ENV{'PATH'});
+         foreach my $dvar(@dvars) {
+            chomp($dvar);
+            $gnBinPath = $dvar;
+            if(-e "$gnBinPath\/perl") {
+               return($gnBinPath);
+            }
+         }
+      }
+      my @dvars = ('/usr/local/bin/', '/usr/bin/');
+      foreach my $dvar(@dvars) {
+         chomp($dvar);
+         $gnBinPath = $dvar;
+         if(-e "$gnBinPath\/perl") {
+            return($gnBinPath);
+         }
+      }
+      return($gnBinPath);
+}
+
+sub gnGetLibPath {
+   # ライブラリパスをチェック
+   #
+   # 引数:      なし
+   # 返り値: 1: ライブラリへのフルパス
+   
+   my $gnLibPath;
+   if ($^O eq "MSWin32") {
+      # "perl -V"の"osname"がMSWin32の場合は
+      # 実行環境がWindowsであると判断
+      $gnLibPath = "$ENV{'APPDATA'}" . "\\gnview\\";
+   }else{
+      if($ENV{'XDG_DATA_DIRS'}) {
+         my @dvars = split(/:/, $ENV{'XDG_DATA_DIRS'});
+         foreach my $dvar(@dvars) {
+            chomp($dvar);
+            $gnLibPath = $dvar;
+            if(-d "$gnLibPath\/applications") {
+               return($gnLibPath);
+            }
+         }
+      }
+      my @dvars = ('/usr/local/share/', '/usr/share/');
+      foreach my $dvar(@dvars) {
+         chomp($dvar);
+         $gnLibPath = $dvar;
+         if(-d "$gnLibPath\/applications") {
+            return($gnLibPath);
+         }
+      }
+   }
+      return($gnLibPath);
+   
+}
+
+sub gnWhich {
+   # 環境変数からperlバイナリの場所を特定する関数
+   # 返り値=配列(ここの要素は文字列。perlへのフルパス)
+   my @path;
+   my @retpath;
+   @path = split(/:/, $ENV{'PATH'});
+   foreach my $pt(@path) {
+      if(-e "$pt\/perl") {
+         push(@retpath, "$pt\/perl");
+      }
+   }
+   return(@retpath);
+   
+}
+
+sub gnMkdir {
+   # ディレクトリを作成する関数
+   # 引数=文字列(作成するディレクトリ)
+   # 引数2=文字列(パーミッション。半角数字4文字を想定)
+   # 返り値=integer(1=成功、2=失敗:既にある、3=失敗:作成に失敗)
+   my $dir = $_[0];
+   my $perm = $_[1];
+      if(!(-d $dir)) {
+          my $flg;
+          $flg = mkpath($dir, 0, oct($perm));
+          if ($flg == 0) {
+             &gnprint("ディレクトリの作成に失敗しました: $dir\n");
+             return(3);
+            }else{
+               &gnprint("ディレクトリを作成しました: $dir\n");
+               return(1);
+            }
+      }else{
+         &gnprint("ディレクトリは既に存在します: $dir\n");
+         return(2);
+       }
+}
+
+sub gnInstall {
+    # 引数1=配列(ここの要素は文字列。コピーするファイル名が入っていると想定)
+    # 引数2=文字列(Permission。ファイルをコピーしたのちにこのパーミッションを割り当てる)
+    # 引数3=文字列(path。コピー先のディレクトリ)
+    my @mykeys = @_;
+    my $destdir = pop(@mykeys);
+    my $perm = pop(@mykeys);
+    foreach my $fls(@mykeys) {
+       copy($fls, $destdir);
+       my $uname = $ENV{'LOGNAME'};
+       my ($login, $pass, $uid, $gid) = getpwnam($uname);
+       chown($uid, $gid, "${destdir}\/${fls}");
+       chmod(oct($perm), "${destdir}\/${fls}");
+       &gnprint("${fls}\t=\> ${destdir}\/${fls}\n");
+    }    
+}
+
 sub gnprint {
     # ロケール名に合わせて文字コードを変換し標準出力に出力する
     # 引数=文字列
